;;;
;;; Virtual Disk Drive Project	- Sep 16, 2001 - EJR (in Austin)
;;;

;;; Main routine.

#include "vd.h"
#include "intrupt.h"
#include "serial.h"
#include "disk.h"
#include "hardware.h"
#include "sertrack.h"

	IFDEF __16C74B
    MESSG "BUILDING FOR 16C74B"
	list p=16c74b,f=INHX32	; INHX32 required for config bits in file
	__config 0x3FB2		; xx11 1111 x011 0010
	ENDIF
	IFDEF __16F877
    MESSG "BUILDING FOR 16F877"
	list p=16f877,f=INHX32	; INHX32 required for config bits in file
	__config 0x3F32		; xx11 1111 0011 0010
	ENDIF
	IFDEF __16F884
    MESSG "BUILDING FOR 16F884"
	list p=16f884,f=INHX32	; INHX32 required for config bits in file
	__config _CONFIG1,0x20E2
	__config _CONFIG2,0x0600
	ENDIF
				;   Code protect = off (13-8,5,4)
				;   (not used) (7)
				;   Brown out timer = off (6)
				;   /Power up timer = on (3)
				;   Watchdog = off (2)
				;   Clock = HS (1,0)
	Title "vd.asm"

;;;
;;; This routine exports the following symbols
;;;
 	global	Start
	global	DiskTable
	global	GCR6x2Map
	global	rGCR6x2Map
	global	rGCR6x2MapUnsafe

PROG1	CODE

;;;*******************************************************************
;;; NAME:	DiskTable()
;;;
;;; DESCR:	Disk mapping table.  It was put here to ensure that it
;;;		stayed within a single code page, to make calling it
;;;		easier.
;;;
;;; ARGS:	the disk offset was placed in DTOFFSET
;;;
;;; RETURNS:	the disk pointer is in W
;;;
;;; NOTES:	- have minimal race condition here, it is possible that the
;;; 		  signal changes on the disk select lines during the few cycles
;;;		  that we spend checking for them.
;;;		- 4 cycles chewed up (including return) (+2 for "call")
;;;		- NOTE:	10/1/04 - 5 cycles are actually used, the addwf
;;;		  modifies the PCL, causing it to take 2 cycles
;;;*******************************************************************
DiskTable:
	movf	DTOFFSET,W	; (0)
	addwf	PCL		; (1)
	retlw	NULLDISK	; (2,3) 0,0
	retlw	DISK0		; (2,3) 0,1
	retlw	DISK1		; (2,3) 1,0
	retlw	DISK2		; (2,3) 1,1

;;;*******************************************************************
;;; GCR Tables used for encoding GCR.  These must be at the start of
;;; a 256-byte page so the "jump-table" nature won't blow up.
;;;*******************************************************************
;;;*******************************************************************
;;; NAME:	  GCR6x2Map()   and  rGCR6x2Map()
;;;             * 64 entries *     * 128 entries *
;;;
;;; DESCR:	called to map 6/5 bits to an 8-bit value, and back again
;;;
;;; ARGS:
;;;
;;; RETURNS:
;;;
;;; NOTES:	- these routines MUST NOT cross a 256-byte boundary
;;;		  - and remember that it has 64 bytes
;;;		- takes 7 cycles (2 for call, 2 for "jump",
;;;			2 for return, and 1 for the mask)
;;;		- there used to be GCR5x3 maps, but I've abandoned
;;;		  creating sector-based 5x3 support, opting to let
;;;		  NIB format take care of 5x3.  Look in v2.3 and below
;;;		  for this old code.
;;;		- reverse mapping "bad" combinations return 0x00
;;;		- top bit in reverse GCR6x2 is assumed to be 1
;;; 		     ie - we really only use the last seven bits
;;;		- I considered using an "addlw 0x70" to save 16 slots in
;;;		  the reverse mapping, but this could cause a jump into
;;;		  oblivion with bad input data, and I didn't want to mask
;;;		  first 'cause I didn't want the extra instr cycle
;;;		  (trading instr memory for speed)
;;;*******************************************************************
GCR6x2Map:
	andlw	0x3f		; (2) uses lower 6 bits
	addwf	PCL		; (3)
	retlw	0x96		; (5) 00
	retlw	0x97		; (5) 01
	retlw	0x9A		; (5) 02
	retlw	0x9B		; (5) 03
	retlw	0x9D		; (5) 04
	retlw	0x9E		; (5) 05
	retlw	0x9F		; (5) 06
	retlw	0xA6		; (5) 07
	retlw	0xA7		; (5) 08
	retlw	0xAB		; (5) 09
	retlw	0xAC		; (5) 0A
	retlw	0xAD		; (5) 0B
	retlw	0xAE		; (5) 0C
	retlw	0xAF		; (5) 0D
	retlw	0xB2		; (5) 0E
	retlw	0xB3		; (5) 0F
	retlw	0xB4		; (5) 10
	retlw	0xB5		; (5) 11
	retlw	0xB6		; (5) 12
	retlw	0xB7		; (5) 13
	retlw	0xB9		; (5) 14
	retlw	0xBA		; (5) 15
	retlw	0xBB		; (5) 16
	retlw	0xBC		; (5) 17
	retlw	0xBD		; (5) 18
	retlw	0xBE		; (5) 19
	retlw	0xBF		; (5) 1A
	retlw	0xCB		; (5) 1B
	retlw	0xCD		; (5) 1C
	retlw	0xCE		; (5) 1D
	retlw	0xCF		; (5) 1E
	retlw	0xD3		; (5) 1F
	retlw	0xD6		; (5) 20
	retlw	0xD7		; (5) 21
	retlw	0xD9		; (5) 22
	retlw	0xDA		; (5) 23
	retlw	0xDB		; (5) 24
	retlw	0xDC		; (5) 25
	retlw	0xDD		; (5) 26
	retlw	0xDE		; (5) 27
	retlw	0xDF		; (5) 28
	retlw	0xE5		; (5) 29
	retlw	0xE6		; (5) 2A
	retlw	0xE7		; (5) 2B
	retlw	0xE9		; (5) 2C
	retlw	0xEA		; (5) 2D
	retlw	0xEB		; (5) 2E
	retlw	0xEC		; (5) 2F
	retlw	0xED		; (5) 30
	retlw	0xEE		; (5) 31
	retlw	0xEF		; (5) 32
	retlw	0xF2		; (5) 33
	retlw	0xF3		; (5) 34
	retlw	0xF4		; (5) 35
	retlw	0xF5		; (5) 36
	retlw	0xF6		; (5) 37
	retlw	0xF7		; (5) 38
	retlw	0xF9		; (5) 39
	retlw	0xFA		; (5) 3A
	retlw	0xFB		; (5) 3B
	retlw	0xFC		; (5) 3C
	retlw	0xFD		; (5) 3D
	retlw	0xFE		; (5) 3E
	retlw	0xFF		; (5) 3F

#define BADGCR	0xff

;;; the "Unsafe" entry point saves 1 cycle (6 consumed as opposed to 7)
;;; but assumes that the caller keeps the argument to 7 bits (the top
;;; bit MUST be zero or the results of this routine are undefined

rGCR6x2Map:
	andlw	0x7f		; (2) uses lower 7 bits
	
rGCR6x2MapUnsafe:
	addwf	PCL		; (3)
	retlw	BADGCR		; (5) 0x80 (00)
	retlw	BADGCR		; (5) 0x81 (01)
	retlw	BADGCR		; (5) 0x82 (02)
	retlw	BADGCR		; (5) 0x83 (03)
	retlw	BADGCR		; (5) 0x84 (04)
	retlw	BADGCR		; (5) 0x85 (05)
	retlw	BADGCR		; (5) 0x86 (06)
	retlw	BADGCR		; (5) 0x87 (07)
	retlw	BADGCR		; (5) 0x88 (08)
	retlw	BADGCR		; (5) 0x89 (09)
	retlw	BADGCR		; (5) 0x8A (0A)
	retlw	BADGCR		; (5) 0x8B (0B)
	retlw	BADGCR		; (5) 0x8C (0C)
	retlw	BADGCR		; (5) 0x8D (0D)
	retlw	BADGCR		; (5) 0x8E (0E)
	retlw	BADGCR		; (5) 0x8F (0F)
	retlw	BADGCR		; (5) 0x90 (10)
	retlw	BADGCR		; (5) 0x91 (11)
	retlw	BADGCR		; (5) 0x92 (12)
	retlw	BADGCR		; (5) 0x93 (13)
	retlw	BADGCR		; (5) 0x94 (14)
	retlw	BADGCR		; (5) 0x95 (15)
	retlw	0x00		; (5) 0x96 (16)
	retlw	0x01		; (5) 0x97 (17)
	retlw	BADGCR		; (5) 0x98 (18)
	retlw	BADGCR		; (5) 0x99 (19)
	retlw	0x02		; (5) 0x9A (1A)
	retlw	0x03		; (5) 0x9B (1B)
	retlw	BADGCR		; (5) 0x9C (1C)
	retlw	0x04		; (5) 0x9D (1D)
	retlw	0x05		; (5) 0x9E (1E)
	retlw	0x06		; (5) 0x9F (1F)
	retlw	BADGCR		; (5) 0xA0 (20)
	retlw	BADGCR		; (5) 0xA1 (21)
	retlw	BADGCR		; (5) 0xA2 (22)
	retlw	BADGCR		; (5) 0xA3 (23)
	retlw	BADGCR		; (5) 0xA4 (24)
	retlw	BADGCR		; (5) 0xA5 (25)
	retlw	0x07		; (5) 0xA6 (26)
	retlw	0x08		; (5) 0xA7 (27)
	retlw	BADGCR		; (5) 0xA8 (28)
	retlw	BADGCR		; (5) 0xA9 (29)
	retlw	BADGCR		; (5) 0xAA (2A)
	retlw	0x09		; (5) 0xAB (2B)
	retlw	0x0A		; (5) 0xAC (2C)
	retlw	0x0B		; (5) 0xAD (2D)
	retlw	0x0C		; (5) 0xAE (2E)
	retlw	0x0D		; (5) 0xAF (2F)
	retlw	BADGCR		; (5) 0xB0 (30)
	retlw	BADGCR		; (5) 0xB1 (31)
	retlw	0x0E		; (5) 0xB2 (32)
	retlw	0x0F		; (5) 0xB3 (33)
	retlw	0x10		; (5) 0xB4 (34)
	retlw	0x11		; (5) 0xB5 (35)
	retlw	0x12		; (5) 0xB6 (36)
	retlw	0x13		; (5) 0xB7 (37)
	retlw	BADGCR		; (5) 0xB8 (38)
	retlw	0x14		; (5) 0xB9 (39)
	retlw	0x15		; (5) 0xBA (3A)
	retlw	0x16		; (5) 0xBB (3B)
	retlw	0x17		; (5) 0xBC (3C)
	retlw	0x18		; (5) 0xBD (3D)
	retlw	0x19		; (5) 0xBE (3E)
	retlw	0x1A		; (5) 0xBF (3F)
	retlw	BADGCR		; (5) 0xC0 (40)
	retlw	BADGCR		; (5) 0xC1 (41)
	retlw	BADGCR		; (5) 0xC2 (42)
	retlw	BADGCR		; (5) 0xC3 (43)
	retlw	BADGCR		; (5) 0xC4 (44)
	retlw	BADGCR		; (5) 0xC5 (45)
	retlw	BADGCR		; (5) 0xC6 (46)
	retlw	BADGCR		; (5) 0xC7 (47)
	retlw	BADGCR		; (5) 0xC8 (48)
	retlw	BADGCR		; (5) 0xC9 (49)
	retlw	BADGCR		; (5) 0xCA (4A)
	retlw	0x1B		; (5) 0xCB (4B)
	retlw	BADGCR		; (5) 0xCC (4C)
	retlw	0x1C		; (5) 0xCD (4C)
	retlw	0x1D		; (5) 0xCE (4E)
	retlw	0x1E		; (5) 0xCF (4F)
	retlw	BADGCR		; (5) 0xD0 (50)
	retlw	BADGCR		; (5) 0xD1 (51)
	retlw	BADGCR		; (5) 0xD2 (52)
	retlw	0x1F		; (5) 0xD3 (53)
	retlw	BADGCR		; (5) 0xD4 (54)
	retlw	BADGCR		; (5) 0xD5 (55)
	retlw	0x20		; (5) 0xD6 (56)
	retlw	0x21		; (5) 0xD7 (57)
	retlw	BADGCR		; (5) 0xD8 (58)
	retlw	0x22		; (5) 0xD9 (59)
	retlw	0x23		; (5) 0xDA (5A)
	retlw	0x24		; (5) 0xDB (5B)
	retlw	0x25		; (5) 0xDC (5C)
	retlw	0x26		; (5) 0xDD (5D)
	retlw	0x27		; (5) 0xDE (5E)
	retlw	0x28		; (5) 0xDF (5F)
	retlw	BADGCR		; (5) 0xE0 (60)
	retlw	BADGCR		; (5) 0xE1 (61)
	retlw	BADGCR		; (5) 0xE2 (62)
	retlw	BADGCR		; (5) 0xE3 (63)
	retlw	BADGCR		; (5) 0xE4 (64)
	retlw	0x29		; (5) 0xE5 (65)
	retlw	0x2A		; (5) 0xE6 (66)
	retlw	0x2B		; (5) 0xE7 (67)
	retlw	BADGCR		; (5) 0xE8 (68)
	retlw	0x2C		; (5) 0xE9 (69)
	retlw	0x2D		; (5) 0xEA (6A)
	retlw	0x2E		; (5) 0xEB (6B)
	retlw	0x2F		; (5) 0xEC (6C)
	retlw	0x30		; (5) 0xED (6D)
	retlw	0x31		; (5) 0xEE (6E)
	retlw	0x32		; (5) 0xEF (6F)
	retlw	BADGCR		; (5) 0xF0 (70)
	retlw	BADGCR		; (5) 0xF1 (71)
	retlw	0x33		; (5) 0xF2 (72)
	retlw	0x34		; (5) 0xF3 (73)
	retlw	0x35		; (5) 0xF4 (74)
	retlw	0x36		; (5) 0xF5 (75)
	retlw	0x37		; (5) 0xF6 (76)
	retlw	0x38		; (5) 0xF7 (77)
	retlw	BADGCR		; (5) 0xF8 (78)
	retlw	0x39		; (5) 0xF9 (79)
	retlw	0x3A		; (5) 0xFA (7A)
	retlw	0x3B		; (5) 0xFB (7B)
	retlw	0x3C		; (5) 0xFC (7C)
	retlw	0x3D		; (5) 0xFD (7D)
	retlw	0x3E		; (5) 0xFE (7E)
	retlw	0x3F		; (5) 0xFF (7F)
	

;;;*******************************************************************
;;; NAME:	Start()
;;;
;;; DESCR:	entry point for this thing
;;;
;;; ARGS:
;;;
;;; RETURNS:
;;;
;;; NOTES:
;;;*******************************************************************
Start:
	PAGESEL	HardwareSetup		; zip over to page 1
	call	HardwareSetup		; these two routines are on page 1
	call	InterruptSetup
	PAGESEL	Start			; get back to this page (0)

	call	DiskSetSetup
	call	SerialSetup

	bsf	RUNSTATE,STOPPING
	bsf	RUNSTATE,STOPPED  ; upon reset, go into stopped state

Loop:
	btfss	RUNSTATE,STOPPING  ; if not stopping (implies stopped too)
	call	Running		   ;  then we must be running

	btfsc	RUNSTATE,STOPPING
	bsf	RUNSTATE,STOPPED   ; got a stop command, indicate stopped

	goto	Loop

;;;*******************************************************************
;;; NAME:	Running()
;;;
;;; DESCR:	Called when running.  Checks head movement too.
;;;
;;; ARGS:
;;;
;;; RETURNS:
;;;
;;; NOTES:
;;;*******************************************************************
Running:
	DiskLineMap			; (0) get state of disk select
	call		DiskTable	; (7) afterthis, W has disk selection

	xorlw		NULLDISK	; (13) check to see if a disk was selected
	skpnz				; (14)
	return				; (15)

	xorlw		NULLDISK	; (16) get back to the selected disk

	;; W has the disk to activate

	call		DiskActivate	; and activate the disk

	;; so the given disk was activated, but it may not have been
	;; loaded from the PC, in this case, don't respond to that
	;; drive request!  This is done by checking for a NULLDISK.

	movlw		NULLDISK
	subwf		CURDISK,W
	skpnz				; don't do track in this case
	return

	movf		WRITEPROT
	skpz
	Raise		WPROTECT	; if non-zero WRITEPROT, turn on write-protect

	call		SerTrack	; send out the track
	Lower		TRACK0		; bring down track 0 line in any case

	Lower		WPROTECT	;  lower write protect in any case
	return

;;; =======================================================================
;;; =======================================================================

	END
