;;; 
;;; Virtual Disk Drive Project	- Mar 1, 2002 - EJR (on plane to Austin)
;;;
;;; Utility Functions

#include "vd.h"
	
;;;
;;; This routine exports the following symbols
;;; 
   global Mod256	      ; used during alignment
   global ModInverse          ; used during alignment
   global Multiply	      ; simply does a multiply
   global XmitWait	      ; returns when able to transmit again
   global PrintByte	      ; sends a byte out to the serial port
	
PROG1 CODE
		
;;;*******************************************************************
;;; NAME:	Mod256()
;;;
;;; DESCR:	Divides 256 by the argument in W, and returns the
;;;		remainder.
;;;
;;; ARGS:	W has the divisor
;;;
;;; RETURNS:	W has the remainder.  The quotient is punted.
;;;
;;; NOTES:	- This routine assumes that incoming W is less than 256.
;;;		- WORK1 is used
;;;*******************************************************************
Mod256:
		clrf	WORK1
		subwf	WORK1	;  first subtract

Mod2562:	
		subwf	WORK1
		skpnc		; if rolled over, then prepare return
		goto	Mod2562

		addwf	WORK1,W	; rolled over at this point

		return		; W has the remainder

;;;*******************************************************************
;;; NAME:	ModInverse()
;;;
;;; DESCR:	Divide ARG0 by W, return the the subtraction of the
;;;		divisor and the remainder.  This number is suitable to
;;;		add to the dividend to create a number divisible by
;;;		the divisor.
;;;
;;; ARGS:	W - the divisor (small number)
;;;		ARG0 - the dividend (big number)
;;;
;;; RETURNS:	W - the inverse remainder of the division
;;;
;;; NOTES:	WORK1 is used
;;;*******************************************************************
ModInverse:
		movwf	WORK1	; save a copy of divisor
	
ModI2:	
		subwf	ARG0	; subtract until carry hits
		skpnc
		goto	ModI2

		addwf	ARG0,W	; back up to before failure

		skpnz
		return		; return zero if ARG0 is a multiple of
				;  divisor or was originally zero 

	;; at this point W has remainder, WORK1 has divisor

		subwf	WORK1,W	; find inverse mod and return

		return

;;;*******************************************************************
;;; NAME:	Multiply()
;;;
;;; DESCR:	Multiplies the argument in ARG0 by W.  It is assumed
;;;		in this routine that the result is less than 256.
;;;
;;; ARGS:	ARG0 is the looping argument (header bytes for eg)
;;;		W is the adding arg (number of sectors for eg)
;;;
;;; RETURNS:	result is in W
;;;
;;; NOTES:	- for eg, 18 sectors times 9 header bytes...should loop
;;;		  on the small number...which is probably header bytes.
;;;		- WORK1 is used
;;;		- 0 in the looping arg (ARG0) is interpreted at 256
;;;		- cycles consumed: ARG0 * 3 + 2 (not incl call & return)
;;;*******************************************************************
Multiply:
		clrf	WORK1
Multloop:	
		addwf	WORK1
		decfsz	ARG0
		goto	Multloop
		movf	WORK1,W
		return

;;;*******************************************************************
;;; NAME:	XmitWait()
;;;
;;; DESCR:	Returns only when the serial xmit buffer is available
;;;		for transmitting another byte.
;;;
;;; ARGS:	
;;;
;;; RETURNS:	
;;;
;;; NOTES:
;;;*******************************************************************
XmitWait:
	btfss	PIR1,TXIF	; Check for xmit buffer clear
	goto	XmitWait	; spin until it is
	return

;;;**********************************************************************
;;; NAME:	PrintByte()
;;;
;;; DESCR:	Sends the byte in W out to the serial port.
;;;
;;; ARGS:	
;;;
;;; RETURNS:	
;;;
;;; NOTES:
;;;**********************************************************************
PrintByte:	
	call	XmitWait
        movwf	TXREG
        return
	
;;; ---------------------------------------------------------------------------
	
	END
	